# -*- coding: utf-8 -*-
#
# django_pam/accounts/view_mixins.py
#

"""
Dynamic Column view mixins.
"""
__docformat__ = "restructuredtext en"

import logging

from django.http import JsonResponse

log = logging.getLogger('django_pam.accounts.views')


class JSONResponseMixin(object):
    """
    A mixin that can be used to render a JSON response.
    """

    def render_to_json_response(self, context, **response_kwargs):
        """
        Returns a JSON response, transforming 'context' to make the payload.

        :param context: The template rendering context.
        :type context: See `Django Context <https://docs.djangoproject.com/en/dev/ref/templates/api/#playing-with-context-objects>`_.
        :param response_kwargs: Response keywords arguments.
        :rtype: See `Django response_class <https://docs.djangoproject.com/en/dev/ref/class-based-views/mixins-simple/#django.views.generic.base.TemplateResponseMixin.response_class>`_.
        """
        return JsonResponse(self.get_data(**context), **response_kwargs)

    def get_data(self, **context):
        """
        Returns an object that will be serialized as JSON by json.dumps().

        :param context: Context added to the JSON response.
        :type context: dict
        :rtype: dict -- Updated context
        """
        # Note: This is *EXTREMELY* naive; in reality, you'll need
        # to do much more complex handling to ensure that arbitrary
        # objects -- such as Django model instances or querysets
        # -- can be serialized as JSON.
        return context

class AjaxableResponseMixin(object):
    """
    Mixin to add AJAX support to a form. Must be used with an object-based
    FormView (e.g. CreateView)
    """

    def form_invalid(self, form):
        """
        Renders the invalid form error description. See `Django's form_invalid
        <https://docs.djangoproject.com/en/dev/ref/class-based-views/mixins-editing/#django.views.generic.edit.FormMixin.form_invalid>`_.
        If the request is an AJAX request return this data as a JSON string.

        :param form: The Django form object.
        :type form: Django's form object.
        :rtype: Result from Django's ``form_valid`` or a JSON string.
        """
        response = super(AjaxableResponseMixin, self).form_invalid(form)

        if self.request.is_ajax():
            return JsonResponse(form.errors, status=422)
        else:
            return response

    def form_valid(self, form):
        """
        Renders the valid data. See `Django's form_valid <https://docs.djangoproject.com/en/dev/ref/class-based-views/mixins-editing/#django.views.generic.edit.FormMixin.form_valid>`_.
        If the request is an AJAX request return this data as a JSON string.

        :param form: The Django form object.
        :type form: Django's form object.
        :rtype: Result from Django's ``form_valid`` or a JSON string.
        """
        # We make sure to call the parent's form_valid() method because
        # it might do some processing (in the case of CreateView, it will
        # call form.save() for example).
        response = super(AjaxableResponseMixin, self).form_valid(form)

        if self.request.is_ajax():
            return JsonResponse(self.get_data(**{}))
        else:
            return response

    def get_data(self, **context):
        """
        Returns an object that will be serialized as JSON by json.dumps().

        :param context: Context added to the JSON response.
        :type context: dict
        :rtype: dict -- Updated context
        """
        context.update({'pk': self.object.pk})
        return context



#-*- coding: utf-8 -*-
#
# django_pam/accounts/forms.py
#
"""
Django PAM forms.
"""
__docformat__ = "restructuredtext en"

import logging
import inspect

from django import forms
from django.contrib.auth import authenticate
from django.contrib.auth.forms import AuthenticationForm as _AuthenticationForm

log = logging.getLogger('django_pam.accounts.forms')


class AuthenticationForm(_AuthenticationForm):
    """
    Authentication form
    """
    email = forms.EmailField(required=False, label_suffix='')

    def __init__(self, request=None, *args, **kwargs):
        if log.isEnabledFor(logging.DEBUG):
            debug = kwargs.copy()
            data = dict([(k, 'Has Password' if 'password' in k and v else v)
                         for k, v in debug.get('data', {}).items()])
            debug['data'] = data
            log.debug("request: %s, args: %s, kwargs: %s", request, args, debug)

        self.base_fields['username'].label_suffix = ''
        self.base_fields['password'].label_suffix = ''
        super(AuthenticationForm, self).__init__(
            request=request, *args, **kwargs)

    def clean(self):
        """
        Does the authentication and saves the email if exists.
        """
        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')
        email = self.cleaned_data.get('email')

        if username and password:
            self.user_cache = authenticate(username=username, password=password)

            if self.user_cache:
                if email:
                    self.user_cache.email = email
                    self.user_cache.save()

                self.confirm_login_allowed(self.user_cache)

        if not (username and password and self.user_cache):
            raise forms.ValidationError(
                self.error_messages['invalid_login'],
                code='invalid_login',
                params={'username': self.username_field.verbose_name},
                )

        return self.cleaned_data

    class Media:
        css = {
            'all': ('django_pam/css/auth.css',)
            }
        js = ('django_pam/js/auth.js',)



#-*- coding: utf-8 -*-
#
# django_pam/accounts/views.py
#
"""
Django PAM views.py
"""
__docformat__ = "restructuredtext en"


import logging
import functools
import smtplib
import socket
import json

from django.core.exceptions import ImproperlyConfigured
from django.contrib.auth import (
    REDIRECT_FIELD_NAME, login, logout, get_user_model)
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext, ugettext_lazy as _
from django.utils.encoding import force_text
from django.views.decorators.csrf import csrf_protect
from django.views.decorators.cache import never_cache
from django.views.generic import TemplateView
from django.views.generic.edit import FormView
from django.views.generic.list import BaseListView
from django.views.generic.detail import SingleObjectMixin
from django.shortcuts import redirect, resolve_url
from django.conf import settings

from .forms import AuthenticationForm
from .view_mixins import JSONResponseMixin, AjaxableResponseMixin

log = logging.getLogger('django_pam.accounts.views')


#
# LoginView
#
class LoginView(AjaxableResponseMixin, FormView):
    """
    A class version of django.contrib.auth.views.login.

    Usage::

        url(r'^login/$', LoginView.as_view(
            form_class=MyAuthenticationForm,
            success_url='/my/success/url/',
            redirect_field_name='my-redirect-field-name',
            template_name='your_template.html'
            ), name='login'),
    """
    form_class = AuthenticationForm
    redirect_field_name = REDIRECT_FIELD_NAME
    template_name = 'django_pam/accounts/login.html'
    success_url = ''

    @method_decorator(csrf_protect)
    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        """
        Dispatches the request to the correct HTTP handler method.

        :param request: The Django request object.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :rtype: The proper handler.
        """
        return super(LoginView, self).dispatch(request, *args, **kwargs)

    def get_form_kwargs(self):
        """
        Incoming AJAX data structure from an HTML <form> tag::

          [{'name': 'username', 'value': '<username>'},
           {'name': 'password', 'value': '<password>'},
           {'name': 'next', 'value': '<redirect URI>'}
          ]

        :rtype: dict
        """
        if self.request.is_ajax():
            json_data = json.loads(self.request.body.decode('utf-8'))
            kwargs = {}
            data = {}

            for arg in json_data:
                name = arg.get('name')
                value = arg.get('value')

                if name == self.redirect_field_name:
                    self.success_url = resolve_url(value)
                else:
                    data[name] = value

            kwargs['data'] = data
        else:
            kwargs = super(LoginView, self).get_form_kwargs()

        return kwargs

    def form_valid(self, form):
        """
        The user has provided valid credentials (this was checked in the
        form's is_valid() method).

        :param form: A Django form object.
        :type form: Django Form
        :rtype: Result of ``AjaxableResponseMixin.form_valid``.
        """
        self.object = form.get_user()
        login(self.request, self.object)
        return super(LoginView, self).form_valid(form)

    def get_data(self, **context):
        """
        Add to the JSON context.

        :param context: A json response context.
        :type context: dict
        :rtype: dict
        """
        # Called in form_valid in AjaxableResponseMixin.
        context.update({'username': self.object.get_username(),
                        'full_name': self.object.get_full_name(),
                        self.redirect_field_name: self.get_success_url()})
        return super(LoginView, self).get_data(**context)

    def get_success_url(self):
        """
        Returns a url used for redirection.

        :rtype: str
        """
        if self.success_url:
            redirect_to = self.success_url
        else:
            redirect_to = self.request.GET.get(self.redirect_field_name, '')

        if not redirect_to:
            redirect_to = resolve_url(settings.LOGIN_REDIRECT_URL)

        return redirect_to


#
# LogoutView
#
class LogoutView(JSONResponseMixin, TemplateView):
    """
    A class version of django.contrib.auth.views.logout.

    Usage::

        url(r'^logout/$', LogoutView.as_view(
            template_name='my_template.html',
            success_url='/my/success/url/),
            redirect_field_name='my-redirect-field-name'
            ), name='logout')
    """
    template_name = "django_pam/accounts/logout.html"
    redirect_field_name = REDIRECT_FIELD_NAME
    success_url = settings.LOGIN_URL

    def get(self, request, *args, **kwargs):
        """
        A GET method handler.

        :param request: The Django request object.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :type kwargs: dict
        :rtype: A response object.
        """
        log.debug("request: %s, args: %s, kwargs: %s", request, args, kwargs)

        if not request.user.is_authenticated():
            response = redirect(self.get_success_url())
        else:
            next_page = request.GET.get(self.redirect_field_name, '')
            kwargs[self.redirect_field_name] = next_page
            context = self.get_context_data(**kwargs)
            response = self.render_to_response(context)

        return response

    def post(self, request, *args, **kwargs):
        """
        A POST method handler.

        .. note::

          Incoming AJAX data structure from an HTML <form> tag::

            [{'name': 'next', 'value': '<redirect URI>'}]

        :param request: The Django request object.
        :param args: Positional arguments.
        :param kwargs: Keyword arguments.
        :type kwargs: dict
        :rtype: A response object.
        """
        log.debug("request: %s, args: %s, kwargs: %s", request, args, kwargs)
        next_page = request.POST.get(self.redirect_field_name, '')
        kwargs[self.redirect_field_name] = next_page
        self.success_url = next_page

        if request.user.is_authenticated():
            logout(request)

        if self.request.is_ajax():
            response = self.render_to_json_response({})
        else:
            response = redirect(self.get_success_url())

        return response

    def get_data(self, **context):
        """
        Add to the JSON context.

        :param context: A json response context.
        :type context: dict
        :rtype: dict
        """
        # Called in JSONResponseMixin.
        context = super(LogoutView, self).get_data(**context)
        json_data = json.loads(self.request.body.decode('utf-8'))
        log.debug("json_data: %s", json_data)

        for arg in json_data:
            name = arg.get('name')
            value = arg.get('value')

            if name == self.redirect_field_name:
                context[name] = resolve_url(value)
            else:
                context[name] = value

        log.debug("context: %s, success_url: %s", context, self.success_url)
        return context

    def get_context_data(self, **kwargs):
        """
        Add to the template context.

        :param kwargs: Keyword arguments.
        :type kwargs: dict
        :type kwargs: dict
        :rtype: dict
        """
        context = super(LogoutView, self).get_context_data(**kwargs)
        log.debug("kwargs: %s, context: %s", kwargs, context)
        context.update({
            self.redirect_field_name: kwargs.get(self.redirect_field_name),
            })
        return context

    def get_success_url(self):
        """
        Returns the supplied success URL.

        :rtype: str
        """
        if self.success_url:
            # Forcing possible reverse_lazy evaluation
            url = force_text(self.success_url)
        else:
            raise ImproperlyConfigured(
                _("No URL to redirect to. Provide a success_url."))

        return url



# -*- coding: utf-8 -*-
#
# django_pam/accounts/urls.py
#
"""
Django PAM accounta/urls.py


"""
__docformat__ = "restructuredtext en"


from django.conf.urls import url
from django.views.generic import TemplateView

from .views import LoginView, LogoutView


urlpatterns = [
    url(r'^login/$', LoginView.as_view(), name='login'),
    url(r'^logout/$', LogoutView.as_view(), name='logout'),
    ]



# -*- coding: utf-8 -*-
#
# django_pam/auth/backends.py
#
"""
Django PAM views.
"""
__docformat__ = "restructuredtext en"

import logging
import types
import six
import pam as pam_base

from django.contrib.auth import get_user_model
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.backends import ModelBackend

log = logging.getLogger('django_pam.auth.backends')


class PAMBackend(ModelBackend):
    """
    An implementation of a PAM backend authentication module.
    """
    _pam = pam_base.pam()

    def authenticate(self, username=None, password=None, **extra_fields):
        """
        Authenticate using PAM then get the account if it exists else create
        a new account.

        :param username: The users username. This is a manditory field.
        :type username: str
        :param password: The users password. This is a manditory field.
        :type password: str
        :param extra_fields: Additonal keyword options of any editable field
                             in the user model.
        :rtype: The Django user object.
        """
        log.debug("username: %s, extra_fields: %s", username, extra_fields)
        UserModel = get_user_model()
        user = None

        if self._pam.authenticate(username, password):
            try:
                user = UserModel._default_manager.get_by_natural_key(
                    username=username)
            except UserModel.DoesNotExist:
                user = UserModel._default_manager.create_user(
                    username, **extra_fields)

        return user

    def get_user(self, user_data):
        """
        Get the user by either the ``username``, ``email``, or the ``pk``.

        :param user_data: The username, email or pk.
        :type user: str or int
        :rtype: A Django user object.
        """
        UserModel = get_user_model()
        obj = None

        if user_data is not None and (
            isinstance(user_data, six.integer_types) or
            user_data.isdigit()):
            query = models.Q(pk=user_data)
        elif isinstance(user_data, six.string_types):
            query = models.Q(username=user_data) | models.Q(email=user_data)
        else:
            msg = _("The user argument type should be either an integer "
                    "(valid pk) or a string (username or email), found "
                    "type {}.").format(type(user_data))
            log.error(msg)
            raise TypeError(msg)

        try:
            obj = UserModel._default_manager.get(query)
        except UserModel.DoesNotExist:
            pass

        log.debug("user_data: %s, obj: %s", user_data, obj)
        return obj



import os, sys, re
from setuptools import setup, find_packages

def version():
    regex = r'^(?m){}[\s]*=[\s]*(?P<ver>\d*)$'

    with open(os.path.join(os.path.dirname(__file__), 'include.mk')) as f:
        ver = f.read()

    major = re.search(regex.format('MAJORVERSION'), ver).group('ver')
    minor = re.search(regex.format('MINORVERSION'), ver).group('ver')
    patch = re.search(regex.format('PATCHLEVEL'), ver).group('ver')
    return "{}.{}.{}".format(major, minor, patch)

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-pam',
    version=version(),
    packages=['django_pam', 'django_pam.auth', 'django_pam.accounts',],
    include_package_data=True,
    license='MIT',
    description=('Django PAM can be used in an SSO (Single Sign On) '
                 'environment or just with a single box where you want to '
                 'log into a Django app with your UNIX login.'),
    long_description=README,
    url='https://github.com/cnobile2012/django-pam',
    author='Carl J. Nobile',
    author_email='carl.nobile@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.9',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        ],
    install_requires=[
        'python-pam',
        'six',
        ],
    )



# -*- coding: utf-8 -*-
#
# django_pam/urls.py
#
"""
Django PAM urls.py
"""
__docformat__ = "restructuredtext en"


from django.conf.urls import include, url


urlpatterns = [
    url(r'^', include('django_pam.accounts.urls', namespace='django-pam')),
    ]
